﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("AnyNodeSymbol", "A symbol that represents a '=' wildcard that can match any node of the same type (function or leaf)")]
  public class AnySubtreeSymbol : Symbol {
    public AnySubtreeSymbol(bool deserializing) : base(deserializing) { }

    public AnySubtreeSymbol(Symbol original, Cloner cloner) : base(original, cloner) { }

    public AnySubtreeSymbol(string name, string description) : base(name, description) { }

    protected AnySubtreeSymbol(AnySubtreeSymbol original, Cloner cloner) : base(original, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AnySubtreeSymbol(this, cloner);
    }

    public AnySubtreeSymbol() : base("#", "A wildcard symbol that can match any valid subtree") { }

    public override ISymbolicExpressionTreeNode CreateTreeNode() {
      return new AnySubtree(this);
    }

    public override int MinimumArity { get; }
    public override int MaximumArity { get; }
  }
}
